#
# raccp.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Library General Public License.
#

require 'racc/parser'
require 'amstd/symbol'


module Racc

  class RaccParser < Parser

    def initialize( racc )
      @racc = racc
      @interf    = racc.interf
      @ruletable = racc.ruletable
    end


    attr :classname

    attr :filename

    def parse( str, fname, mydebug = false )
      fname.must String
      
      @scanner = RaccScanner.new( str )
      @scanner.debug    = @yydebug  = mydebug
      @scanner.filename = @filename = fname

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      case val
      when Symbol then v = val.id2name
      when String then v = val
      else
        v = val.inspect
      end
      raise ParseError,
%=parse error:#{@filename}:#{@scanner.lineno}: unexpected token '#{v}'=
    end

##### racc 0.13.0 generates ###

Racc_reduce_table = [
 0, 0, :racc_error,
 6, 20, :_reduce_1,
 0, 21, :_reduce_none,
 2, 21, :_reduce_none,
 3, 23, :_reduce_4,
 1, 23, :_reduce_none,
 2, 23, :_reduce_6,
 2, 24, :_reduce_7,
 3, 24, :_reduce_8,
 3, 25, :_reduce_9,
 3, 25, :_reduce_10,
 1, 27, :_reduce_none,
 2, 27, :_reduce_none,
 2, 28, :_reduce_13,
 2, 28, :_reduce_14,
 2, 28, :_reduce_15,
 1, 29, :_reduce_16,
 2, 29, :_reduce_17,
 1, 26, :_reduce_18,
 1, 26, :_reduce_19,
 2, 22, :_reduce_none,
 3, 22, :_reduce_none,
 3, 30, :_reduce_22,
 3, 30, :_reduce_23,
 0, 31, :_reduce_24,
 2, 31, :_reduce_25,
 1, 32, :_reduce_26,
 1, 32, :_reduce_27,
 1, 32, :_reduce_28,
 2, 32, :_reduce_29 ]

Racc_reduce_n = 30

Racc_shift_n = 52

Racc_action_table = [
     2,     1,    -1,   -30,     3,     3,    -1,   -30,     0,     4,
    -1,   -30,    -1,    -2,     0,     6,    -1,   -30,     4,    13,
     6,    11,     7,     9,     9,     8,    10,     7,    -1,   -30,
    -1,    52,    11,    16,    12,    15,    13,    14,    -1,   -30,
    11,    16,    12,    15,    13,    14,    -1,   -30,     3,    20,
    -1,   -30,    -1,    -5,     3,    22,     8,    21,    -1,   -30,
    -1,    -3,     3,    25,    -1,   -30,     3,    22,     8,    21,
    -1,   -30,     3,    22,     8,    21,    -1,   -30,     3,    22,
     8,    21,    -1,   -30,    -1,   -11,     9,    33,    11,    16,
    12,    15,    13,    14,    -1,   -30,    10,    34,    11,    16,
    12,    15,    13,    14,    -1,   -30,    -1,    -6,    -1,   -19,
    -1,   -18,     8,    35,    -1,   -30,     5,    37,     3,    22,
     8,    21,    -1,   -30,    15,    38,    -1,   -30,    14,    39,
    16,    40,    -1,   -30,     5,    42,     3,    25,    -1,   -30,
    -1,   -16,     3,    22,     8,    21,    -1,   -15,     3,    22,
     8,    21,    -1,   -14,     3,    22,     8,    21,    -1,   -13,
    -1,   -12,    -1,   -10,    -1,    -9,    -1,    -7,     8,    44,
    -1,   -30,    -1,    -4,    -1,   -24,    -1,   -20,    -1,   -24,
    14,    47,    16,    40,    -1,   -30,    -1,    -1,    -1,   -17,
    -1,    -8,    18,    48,     3,    51,     8,    50,    17,    49,
    -1,   -22,    18,    48,     3,    51,     8,    50,    17,    49,
    -1,   -23,    -1,   -21,     3,    22,     8,    21,    -1,   -30,
    -1,   -28,    -1,   -27,    -1,   -26,    -1,   -25,    -1,   -29 ]

Racc_action_table_ptr = [
     0,     4,     8,    12,    14,    18,    30,    32,    40,    48,
    52,    54,    60,    62,    66,    72,    78,    84,    86,    96,
   106,   108,   110,   112,   116,   124,   128,   134,   140,   142,
   148,   154,   160,   162,   164,   166,   168,   172,   174,   176,
   178,   180,   186,   188,   190,   192,   202,   212,   214,   220,
   222,   224,   226,   228 ]


Racc_goto_table = [
    20,     2,    21,     5,    25,    10,    23,    12,    28,    17,
    27,    18,    28,    17,    27,    19,    26,    23,    24,    24,
    30,    26,    22,    27,    26,    28,    29,    29,    26,    28,
    29,    30,    26,    28,    29,    31,    28,    32,    28,    32,
    26,    36,    30,    41,    26,    43,    26,    43,    26,    43,
    31,    45,    31,    46,    32,    52,    32,    52,    26,    53,
    -1,    -1 ]

Racc_goto_table_ptr = [
     0,    -1,    -1,     2,    -1,     4,    -1,     8,    12,    -1,
    -1,    16,    -1,    20,    24,    28,    32,    -1,    36,    38,
    -1,    -1,    -1,    -1,    40,    -1,    -1,    42,    -1,    44,
    46,    48,    -1,    -1,    -1,    -1,    -1,    -1,    50,    -1,
    52,    -1,    -1,    -1,    -1,    54,    56,    -1,    58,    -1,
    -1,    -1,    -1,    -1 ]


Racc_token_table = {
 false => 0,
 Object.new => 1,
 :CLASS => 2,
 :TOKEN => 3,
 :RULE => 4,
 :XEND => 5,
 :XTOKEN => 6,
 :START => 7,
 :STRING => 8,
 :PRECHIGH => 9,
 :PRECLOW => 10,
 :LEFT => 11,
 :RIGHT => 12,
 :NONASSOC => 13,
 ";" => 14,
 ":" => 15,
 "|" => 16,
 :ACTION => 17,
 "=" => 18 }

Racc_debug_parser = false

##### racc system variables end #####

 # reduce 0 omitted

 module_eval( '', '(build)', 88 ) {
  def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = val[1].id2name
   result
  end
 }

 # reduce 2 omitted

 # reduce 3 omitted

 module_eval( '', '(build)', 99 ) {
  def _reduce_4( val, _values, result )
      @interf.end_register_conv
   result
  end
 }

 # reduce 5 omitted

 module_eval( '', '(build)', 105 ) {
  def _reduce_6( val, _values, result )
      @interf.register_start @interf.get_token(val[1])
   result
  end
 }

 module_eval( '', '(build)', 110 ) {
  def _reduce_7( val, _values, result )
      @interf.register_conv( val[0], val[1] )
   result
  end
 }

 module_eval( '', '(build)', 114 ) {
  def _reduce_8( val, _values, result )
      @interf.register_conv( val[1], val[2] )
   result
  end
 }

 module_eval( '', '(build)', 119 ) {
  def _reduce_9( val, _values, result )
      @interf.end_register_prec( true )
   result
  end
 }

 module_eval( '', '(build)', 123 ) {
  def _reduce_10( val, _values, result )
      @interf.end_register_prec( false )
   result
  end
 }

 # reduce 11 omitted

 # reduce 12 omitted

 module_eval( '', '(build)', 133 ) {
  def _reduce_13( val, _values, result )
      @interf.register_prec( :Left, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 137 ) {
  def _reduce_14( val, _values, result )
      @interf.register_prec( :Right, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 141 ) {
  def _reduce_15( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 146 ) {
  def _reduce_16( val, _values, result )
      result = val
   result
  end
 }

 module_eval( '', '(build)', 150 ) {
  def _reduce_17( val, _values, result )
      result.push val[1]
   result
  end
 }

 module_eval( '', '(build)', 155 ) {
  def _reduce_18( val, _values, result )
      result = @interf.get_token( val[0] )
   result
  end
 }

 module_eval( '', '(build)', 159 ) {
  def _reduce_19( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
   result
  end
 }

 # reduce 20 omitted

 # reduce 21 omitted

 module_eval( '', '(build)', 169 ) {
  def _reduce_22( val, _values, result )
      @interf.register_rule @interf.get_token(val[0]), val[2]
   result
  end
 }

 module_eval( '', '(build)', 173 ) {
  def _reduce_23( val, _values, result )
      @interf.register_rule nil, val[2]
   result
  end
 }

 module_eval( '', '(build)', 178 ) {
  def _reduce_24( val, _values, result )
      result = []
   result
  end
 }

 module_eval( '', '(build)', 182 ) {
  def _reduce_25( val, _values, result )
      result.push val[1] if val[1]
   result
  end
 }

 module_eval( '', '(build)', 187 ) {
  def _reduce_26( val, _values, result )
      result = @interf.get_token( val[0] )
   result
  end
 }

 module_eval( '', '(build)', 191 ) {
  def _reduce_27( val, _values, result )
      result = @interf.get_token( eval %<"#{result}"> )
   result
  end
 }

 module_eval( '', '(build)', 195 ) {
  def _reduce_28( val, _values, result )
      result = Action.new( *result )
   result
  end
 }

 module_eval( '', '(build)', 199 ) {
  def _reduce_29( val, _values, result )
      @interf.register_tmpprec val[1]
      result = nil
   result
  end
 }

  end   # class RaccParser

end   # module Racc
